'use strict';

const jsxString = require('jsx-string');
const isURL = require('validator/lib/isURL');
const isISO8601 = require('validator/lib/isISO8601');

const xmlStringify = content => `<?xml version="1.0" encoding="utf-8"?>${jsxString(content)}`;

function createSitemapindex(sitemaps = []) {
  const sitemap = ({ loc, lastmod }) => jsxString.h(
    'sitemap',
    null,
    jsxString.h(
      'loc',
      null,
      loc
    ),
    lastmod && jsxString.h(
      'lastmod',
      null,
      lastmod
    )
  );

  const index = jsxString.h(
    'sitemapindex',
    { xmlns: 'http://www.sitemaps.org/schemas/sitemap/0.9' },
    sitemaps.map(sitemap)
  );

  return {
    stringify: () => xmlStringify(index)
  };
}

const isValidURL = ({ loc, lastmod, changefreq, priority, alternates }) => {
  // loc
  // eslint-disable-next-line camelcase
  if (!loc && !isURL(loc, { require_valid_protocol: true })) {
    throw new Error(`loc "${loc}" was not valid. It's required.

see https://www.sitemaps.org/protocol.html`);
  }

  // lastmod
  if (lastmod !== undefined && !isISO8601(lastmod)) {
    throw new Error(`lastmod "${lastmod}" is not valid. It should be a valid ISO 8601 date

Try using new Date().toISOString()

see https://www.sitemaps.org/protocol.html
see https://www.w3.org/TR/NOTE-datetime`);
  }

  // changefreq
  const allowedFreqs = ['always', 'hourly', 'daily', 'weekly', 'monthly', 'yearly', 'never'];

  if (!allowedFreqs.includes(changefreq) && changefreq !== undefined) {
    throw new Error(`changefreq "${changefreq}" was not a valid value.

expected: ${allowedFreqs.join(', ')}

see https://www.sitemaps.org/protocol.html for more information`);
  }

  // priority
  if (priority < 0 || priority > 1 || !Number.isFinite(priority) && priority !== undefined) {
    throw new Error(`priority "${priority}" was not valid. A number between 0 and 1 is expected.

see https://www.sitemaps.org/protocol.html for more information`);
  }

  // alternates
  const _alternatesError = `alternates ${JSON.stringify(alternates)} was not valid. An object with

{
  languages: ['nl-BE', 'fr', 'pt-BR'],
  hitToURL: (language) => \`the url for that \${language}\`,
}

was expected.`;

  if (alternates !== undefined) {
    if (!(alternates.languages instanceof Array)) {
      throw new Error(_alternatesError);
    }
    if (typeof alternates.hitToURL !== 'function') {
      throw new Error(_alternatesError);
    }
    alternates.languages.forEach(lang => {
      if (!isURL(alternates.hitToURL(lang))) {
        throw new Error(_alternatesError);
      }
    });
  }

  return {
    loc,
    lastmod,
    changefreq,
    priority: priority === undefined ? undefined : priority.toFixed(1),
    alternates
  };
};

function createSitemap(entries = []) {
  const _alternates = ({ languages, hitToURL }) => languages.map(language => ({ language, href: hitToURL(language) })).map(({ language, href }) => jsxString.h('xhtml_link', { rel: 'alternate', hreflang: language, href: href }));

  const url = args => {
    var _isValidURL = isValidURL(args),
        _isValidURL$loc = _isValidURL.loc;

    const loc = _isValidURL$loc === undefined ? undefined : _isValidURL$loc;
    var _isValidURL$lastmod = _isValidURL.lastmod;
    const lastmod = _isValidURL$lastmod === undefined ? undefined : _isValidURL$lastmod;
    var _isValidURL$changefre = _isValidURL.changefreq;
    const changefreq = _isValidURL$changefre === undefined ? undefined : _isValidURL$changefre;
    var _isValidURL$priority = _isValidURL.priority;
    const priority = _isValidURL$priority === undefined ? undefined : _isValidURL$priority;
    var _isValidURL$alternate = _isValidURL.alternates;
    const alternates = _isValidURL$alternate === undefined ? undefined : _isValidURL$alternate;

    return jsxString.h(
      'url',
      {
        xmlns_xhtml: alternates ? 'http://www.w3.org/1999/xhtml' : undefined
      },
      jsxString.h(
        'loc',
        null,
        loc
      ),
      lastmod && jsxString.h(
        'lastmod',
        null,
        lastmod
      ),
      changefreq && jsxString.h(
        'changefreq',
        null,
        changefreq
      ),
      priority && jsxString.h(
        'priority',
        null,
        priority
      ),
      alternates && _alternates(alternates)
    );
  };

  const sitemap = jsxString.h(
    'urlset',
    { xmlns: 'http://www.sitemaps.org/schemas/sitemap/0.9' },
    entries.map(url)
  );

  return {
    stringify: () => xmlStringify(sitemap)
  };
}

module.exports = {
  createSitemapindex,
  createSitemap
};